package com.ym.mec.biz.service.impl;

import com.ym.mec.biz.dal.dao.MusicGroupDao;
import com.ym.mec.biz.dal.dao.SysConfigDao;
import com.ym.mec.biz.dal.entity.CooperationOrgan;
import com.ym.mec.biz.dal.entity.MusicGroup;
import com.ym.mec.biz.dal.entity.SysAccount;
import com.ym.mec.biz.dal.enums.PaymentChannelTypeEnum;
import com.ym.mec.biz.service.PayService;
import com.ym.mec.biz.service.StudentPaymentOrderService;
import com.ym.mec.biz.service.SysAccountService;
import com.ym.mec.thirdparty.adapay.Pay;
import com.ym.mec.thirdparty.union.UnionPay;
import com.ym.mec.thirdparty.union.UnionPayFeignService;
import com.ym.mec.thirdparty.yqpay.YqPayUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

/**
 * 支付服务类
 */
@Service
public class PayServiceImpl implements PayService {

    @Autowired
    private UnionPayFeignService unionPayFeignService;
    @Autowired
    private StudentPaymentOrderService studentPaymentOrderService;
    @Autowired
    private SysAccountService sysAccountService;
    @Autowired
    private MusicGroupDao musicGroupDao;
    @Autowired
    private SysConfigDao sysConfigDao;


    /**
     * 返回订单提交数据
     *
     * @param amount       金额
     * @param orderNo      订单编号
     * @param notifyUrl    异步通知地址
     * @param returnUrl    支付返回地址
     * @param orderSubject 订单标题
     * @param orderBody    订单内容
     * @return
     * @throws Exception
     */
    public Map<String, Object> getPayMap(BigDecimal amount, String orderNo, String notifyUrl, String returnUrl, String orderSubject, String orderBody, Integer userId) throws Exception {

        String company = "daya";
        if (userId != null) {
            MusicGroup musicGroup = musicGroupDao.findUserMusicGroup(userId);
            if (musicGroup.getOwnershipType().equals(CooperationOrgan.OwnershipType.COOPERATION)) {
                company = "yadie";
            }
        }

        //支付通道决策
        Map unionPay = new HashMap();
        SysAccount routingAccount = getRoutingAccount(company, amount);
        PaymentChannelTypeEnum paymentChannelType = PaymentChannelTypeEnum.COM;
        if(routingAccount.getType().equals("PER")){
            paymentChannelType = PaymentChannelTypeEnum.PER;
        }
        Map payMap = null;
        String type = null;
        if (routingAccount.getChannel().equals("YQPAY")) {
            payMap = YqPayUtil.getPayMap(amount, orderNo, notifyUrl, returnUrl, orderSubject, orderBody, routingAccount.getMerNo(), routingAccount.getRoutingMerNo());
            type = "YQPAY";
        } else {
            payMap = new Pay().getPayMap(amount, orderNo, notifyUrl, orderSubject, orderBody);
            type = "ADAPAY";
        }

        unionPay.put("orderNo", orderNo);
        unionPay.put("type", type);
        unionPay.put("paymentChannelType", paymentChannelType);
        unionPay.put("payMap", payMap);
        return unionPay;
    }

    @Override
    public Map<String, Object> query(String orderNo) throws Exception {
        return new UnionPay(unionPayFeignService).query(orderNo);
    }

    /**
     * 获取收款账户
     *
     * @param company
     * @param money
     * @return
     */
    private SysAccount getRoutingAccount(String company, BigDecimal money) {
        SysAccount routingAccount = null;
        int payOrderNums = studentPaymentOrderService.findPayOrderNum(); //获取支付中和成功的订单数
        //前几笔收入私户
        Integer nums = Integer.parseInt(sysConfigDao.findConfigValue("per_account_nums"));

        String type = null;
        String channel = null;
        int rem = (payOrderNums + 1) % 10;
        if (rem > 0 && rem <= nums) { //私人账户
            channel = sysConfigDao.findConfigValue("per_account_channel");
            type = "PER";
        } else {
            channel = sysConfigDao.findConfigValue("com_account_channel");
            type = "COM";
        }
        if (company.equals("yadie")) {
            type = "PER";
        }
        routingAccount = sysAccountService.getAccount(company, channel, type, null);

        return routingAccount;
    }

}
